<?php

namespace Radar\DataBase;

use PDO,
    PDOException;

class DataBase
{

    /**
     * @var \PDO 
     */
    public $dbh;

    /**
     * Конструктор класса
     * @param string $dbHost Хост базы данных
     * @param string $dbLogin Пользователь базы данных
     * @param string|int $dbPassword Пароль пользователя
     */
    public function __construct($dbHost, $dbLogin, $dbPassword)
    {
        $this->setConnection($dbHost, $dbLogin, $dbPassword);
    }

    /**
     * Устанавливает соединение с базой данных
     * @param string $dbHost Хост базы данных
     * @param string $dbLogin Пользователь базы данных
     * @param string|int $dbPassword Пароль пользователя
     * @return boolean Если удалось установить соединение
     */
    public function setConnection($dbHost, $dbLogin, $dbPassword)
    {
        try {
            $options = [
                PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
                PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
                PDO::MYSQL_ATTR_INIT_COMMAND => 'SET NAMES utf8'
            ];

            $this->dbh = new PDO($dbHost, $dbLogin, $dbPassword, $options);
        } catch (PDOException $e) {
            die('Cannot set connection to database :' . $e->getMessage());
        }

        return true;
    }

    /**
     * Закрывает соединение с базой данных
     * @return boolean true Если удалось закрыть, false , если нет соединения
     */
    public function closeConnection()
    {
        if (isset($this->dbh)) {
            unset($this->dbh);
        } else {
            return false;
        }

        return true;
    }

    /**
     * Установлено ли соединение с базой данных
     * @return boolean true если установлено,false если нет
     */
    public function hasConnection()
    {
        if (isset($this->dbh)) {
            return true;
        }

        return false;
    }

    /**
     * 
     * @param array $fields
     * @param array $tables
     * @param string $conditions
     */
    public function update(array $fields, array $tables, $conditions = '')
    {
        $sql = "UPDATE ";
        $sql .= $this->joinTables($tables);
        $sql .= " SET ";
        $sql .= $this->createFieldsNamesValuesString($fields);

        if (!empty($conditions)) {
            $sql .= " WHERE 1 AND ";
            $sql .= $conditions;
        }

        try {
            $res = $this->dbh->exec($sql);
        } catch (PDOException $e) {
            throw new \Radar\Exceptions\UpdateDatasetException($e->getMessage(), $e->getTrace());
        }

        return $res;
    }

    /**
     * 
     * @param array $fields
     * @param array $tables
     * @param string $conditions
     * @param boolean $fetchAll
     * @param boolean $returnState
     * @return array|boolean
     * @throws \Radar\Exceptions\SelectDataException
     */
    public function select(array $fields, array $tables, $conditions = '', $fetchAll = false, $returnState = false)
    {
        $sql = "SELECT ";
        $sql .= $this->createFieldsNamesString($fields);
        $sql .= " FROM ";
        $sql .= $this->joinTables($tables);

        if (!empty($conditions)) {
            $sql .= " WHERE 1 AND ";
            $sql .= $conditions;
        }

        try {
            $res = $this->dbh->query($sql);
        } catch (PDOException $e) {
            throw new \Radar\Exceptions\SelectDataException($e->getMessage(), $e->getTrace());
        }

//  Если вдруг будет нужно вернуть объект PDOStatement
        if ($returnState) {
            return $res;
        }

        if ($fetchAll) {
            $result = $res->fetchAll();
        } else {
            $result = $res->fetch();
        }

        return $result;
    }

    /**
     * 
     * @param aray $fields
     * @param array $values
     * @param string $table
     * @param string $additional
     * @throws \Radar\Exceptions\UnexpectedValueException
     * @throws \Radar\Exceptions\InsertDataException
     */
    public function insert(array $fields, array $values, $table, $additional = '')
    {
        $sql = "INSERT INTO `$table` ";

        if (empty($fields)) {
            throw new \Radar\Exceptions\UnexpectedValueException('Values array must not be empty !');
        }

        $sql .= $this->createFieldsNamesString($fields, true);
        $sql .= " VALUES ";

        $i = 0;
        $fieldsLength = count($values);
        foreach ($values as $varVals) {
            $c = 0;
            $varValsLength = count($varVals);
            foreach ($varVals as $varVal) {
                if ($c === 0) {
                    $sql .= "(";
                }

                $sql .= $this->dbh->quote($varVal);

                if ($c < ($varValsLength - 1)) {
                    $sql .= ",";
                } elseif ($c == ($varValsLength - 1)) {
                    $sql .= ")";
                }

                $c++;
            }

            if ($i < ($fieldsLength - 1)) {
                $sql .= ",\n";
            }

            $i++;
        }

        try {
            $res = $this->dbh->exec($sql);
        } catch (PDOException $e) {
            throw new \Radar\Exceptions\InsertDataException($e->getMessage(), $e->getTrace());
        }

        return $res;
    }

    /**
     * Выбирает строку из базы , используя PDO::Prepare 
     * @param array $fields Поля, которые необходимо выбрать
     * @param array $tables Таблица из которой выбираются данные
     * @param array $conditions Условия по которым будут выбраны данные (название ячейки массива должно соответствовать нававнию колонки в базе)
     * @param boolean $fetchAll Фетчить все строки 
     * @return array $data
     * @throws \Radar\Exceptions\SelectDataException Если не получилось выполнить запрос
     */
    public function selectWithPrepare(array $fields, array $tables, array $conditions = [], $fetchAll = false)
    {
        if (!empty($conditions)) {
            $condition = 'WHERE 1 AND';
            $arExecute = [];

            $arExecute += $this->createFieldsValuesArrayForPreparedRequests($conditions)['varsArray'];
            $condition .= $this->createFieldsValuesArrayForPreparedRequests($conditions)['varsString'];
        }

        $tablesString = $this->joinTables($tables);
        $fieldsString = $this->createFieldsNamesString($fields);

        $sql = "SELECT {$fieldsString} from {$tablesString} {$condition}";

        try {
            $sth = $this->dbh->prepare($sql);
            $sth->execute($arExecute);
        } catch (PDOException $e) {
            throw new \Radar\Exceptions\SelectDataException($e->getMessage(), $e->getTrace());
        }

        if ($fetchAll) {
            $data = $sth->fetchAll();
        } else {
            $data = $sth->fetch();
        }

        return $data;
    }

    /**
     * Вставляет новую строку в базу , используя PDO::Prepare 
     * @param array $values Значения для вставки
     * @param string $table Таблица для вставки
     * @throws \Radar\Exceptions\InsertDataException
     */
    public function insertWithPrepare(array $values, $table)
    {
        $arForInsert = [];
        $insertString = '';
        $arForInsert += $this->createFieldsValuesArrayForPreparedRequests($values)['varsArray'];
        $insertString .= $this->createFieldsValuesArrayForPreparedRequests($values)['varsString'];

        $sql = "Insert into `{$table}` set {$insertString}";

        try {
            $sth = $this->dbh->prepare($sql);
            $sth->execute($arForInsert);
        } catch (PDOException $e) {
            throw new \Radar\Exceptions\InsertDataException($e->getMessage(), $e->getTrace());
        }
    }

    /**
     * Обновляет значения в базе , используя PDO::Prepare
     * @param array $values Значения для записи в базу
     * @param string $table Таблица для апдейта
     * @param array $conditions Условия апдейта
     */
    public function updateWithPrepare(array $values, $table, $conditions = [])
    {
        if (empty($values)) {
            throw new \Radar\Exceptions\UnexpectedValueException('Values array must not be empty !');
        }

        $arForUpdate = [];
        $updateString = '';
        $condition = '';
        $arForUpdate += $this->createFieldsValuesArrayForPreparedRequests($values)['varsArray'];
        $updateString .= $this->createFieldsValuesArrayForPreparedRequests($values)['varsString'];

        if (!empty($conditions)) {
            $condition = ' WHERE ';
            $arForUpdate += $this->createFieldsValuesArrayForPreparedRequests($conditions)['varsArray'];
            $condition .= $this->createFieldsValuesArrayForPreparedRequests($conditions)['varsString'];
        }

        $sql = "Update `{$table}` set {$updateString} {$condition}";

        try {
            $sth = $this->dbh->prepare($sql);
            $sth->execute($arForUpdate);
        } catch (PDOException $e) {
            throw new \Radar\Exceptions\InvalidArgumentException($e->getMessage(), $e->getTrace());
        }
    }

    public function getConnectionHandler()
    {
        return $this->dbh;
    }

    public function joinTables(array $tables, $joinType = 'LEFT')
    {
        if (empty($tables)) {
            throw new \Radar\Exceptions\UnexpectedValueException('Array with tables names must not be empty !');
        }

        $sql = "";
        $i = 0;
        foreach ($tables as $tableName) {
            if ($i > 0) {
                $sql .= "$joinType JOIN ";
            }

            $sql .= " `$tableName` ";

            if ($i > 0) {
                $sql .= " USING(`id`)";
            }

            $i++;
        }

        return $sql;
    }

    /**
     * Обертка для \PDO::beginTransaction
     * @return boolean
     */
    public function beginTransaction()
    {
        return $this->dbh->beginTransaction();
    }

    /**
     * Обертка для PDO::commit
     * @return boolean
     */
    public function commit()
    {
        return $this->dbh->commit();
    }

    /**
     * Обертка для PDO::rollBack
     * @return boolean
     */
    public function rollBack()
    {
        return $this->dbh->rollBack();
    }

    public function createFieldsNamesValuesString(array $fields)
    {
        if (empty($fields)) {
            throw new \Radar\Exceptions\UnexpectedValueException('Array with fields names\vals must not be empty !');
        }

        $sql = "";
        $i = 0;
        $fieldsLength = count($fields);
        foreach ($fields as $fieldName => $fieldValue) {
            $sql .= "`$fieldName` = {$this->dbh->quote($fieldValue)}";

            if ($i < ($fieldsLength - 1)) {
                $sql .= ",";
            }

            $i ++;
        }

        return $sql;
    }

    /**
     * 
     * @param array $fields
     * @param string|boolean $withBrackets
     * @return string
     */
    public function createFieldsNamesString(array $fields, $withBrackets = false)
    {
        $sql = "";

        if ($withBrackets) {
            $sql .= "(";
        }

        $i = 0;
        $varNamesLength = count($fields);
        foreach ($fields as $varName) {
//  '*' Не нужно оборачивать в опострофы
            if ($varName === '*') {
                $sql .= "$varName";
            } else {
                $sql .= "`$varName`";
            }

            if ($i < ($varNamesLength - 1)) {
                $sql .= ",";
            }
            $i++;
        }

        if ($withBrackets) {
            $sql .= ")";
        }

        return $sql;
    }

    private function createFieldsValuesArrayForPreparedRequests(array $arVarsVals)
    {
        if (empty($arVarsVals)) {
            throw new \Radar\Exceptions\UnexpectedValueException('Values array must not be empty !');
        }

        $arVarsValsPairs = [];
        $updateString = '';
        $i = 0;
        $valuesLength = count($arVarsVals);
        foreach ($arVarsVals as $varName => $varValue) {
            $i++;

            if ($i >= $valuesLength) {
                $updateString .= "`{$varName}` = :{$varName}";
            } else {
                $updateString .= "`{$varName}` = :{$varName},";
            }

            $arVarsValsPairs[':' . $varName] = $varValue;
        }

        return ['varsArray' => $arVarsValsPairs, 'varsString' => $updateString];
    }

}
