<?php

namespace Radar\NumbersData;

/**
 * Description of NumbersDataUploader
 *
 * @author Zelepukin
 */
class NumbersDataKeeper
{

    /**
     * @var \Psr\Http\Message\ServerRequestInterface
     */
    private $serverRequest;

    /**
     * @var \Slim\Http\UploadedFile
     */
    private $uploadedFile;

    /**
     * @var \Radar\DataBase\DataBase
     */
    private $db;

    /**
     * Путь для сохранения файла
     * @var string 
     */
    private $pathToSave;

    /**
     * Статус-код при успешной загрузке файла
     */
    const UPLOAD_CODE_SUCCESS = UPLOAD_ERR_OK;

    /**
     * @param \Psr\Http\Message\ServerRequestInterface $request
     * @param \Slim\Http\UploadedFile $file
     * @param \Radar\DataBase\DataBase $db
     * @param string $path
     */
    public function __construct(\Psr\Http\Message\ServerRequestInterface $request, \Radar\DataBase\DataBase $db, $path)
    {
        $this->setPathToSave($path);
        $this->serverRequest = $request;
        $this->db = $db;
    }

    /**
     * Сеттер для переменной с путем сохранения файла 
     * @param string $path
     * @throws \Radar\Exceptions\InvalidArgumentException Если в аргументе пусто или не строка
     */
    public function setPathToSave($path)
    {
        if (empty($path)) {
            throw new \Radar\Exceptions\InvalidArgumentException('Var with path to save file must not be empty !');
        }

        if (!is_string($path)) {
            throw new \Radar\Exceptions\InvalidArgumentException('Var with path to save file must contain a string !');
        }

        $this->pathToSave = $path;
    }

    /**
     * Геттер для переменной с путем сохранения файла.
     * @return string
     */
    public function getPathToSave()
    {
        return $this->pathToSave;
    }

    /**
     * Удалось ли корректно сохранить файл ?
     * @return bool
     */
    public function isFileUploadSuccessfull()
    {
        return ($this->uploadedFile->getError() == UPLOAD_ERR_OK);
    }

    /**
     * Возвращает расширение загружаемого файла 
     * @return bool
     */
    public function getUploadedFileExtension()
    {
        return explode('.', $this->uploadedFile->getClientFilename())[1];
    }

    /**
     * Возвращает название загружаемого файла
     * @return bool
     */
    public function getUploadedFileName()
    {
        return explode('.', $this->uploadedFile->getClientFilename())[0];
    }

    /**
     * Возвращаen ПОЛНЫЙ путь сохранения файла, включая имя файла и расширение
     * @return bool
     */
    public function getPathToSavedFile()
    {
        $fileName = $this->uploadedFile->getClientFilename();
        return $this->pathToSave . $fileName;
    }

    /**
     * Сохраняет загруженнный файл
     * @throws \RuntimeException Если не удалось сохранить файл
     */
    public function saveUploadedFile(\Slim\Http\UploadedFile $file)
    {
        $this->uploadedFile = $file;
        $fullPathToSavedFile = $this->getPathToSavedFile();
        if ($this->isFileUploadSuccessfull()) {
            $this->uploadedFile->moveTo($fullPathToSavedFile);
        } else {
            throw new \RuntimeException('Cannot upload file - ' . $fullPathToSavedFile);
        }
    }

    /**
     * Получает информацию по номерам из массива из базы 
     * @param array $arNumbers
     * @param string $numbersDataTableName Название таблицы в базе с информацией по номерам
     * @param bool $needEncoding Нужно ли менять кодировку ? 
     * @return array
     */
    public function getNumbersInfoFromBase(array $arNumbers, $numbersDataTableName, $needEncoding = true)
    {
        $arNumbersWithoutInfo = $arNumbers;
        $arNumbersWithInfo = [];

        foreach ($arNumbersWithoutInfo as $format) {

            $phoneNumber = $this->getPhoneNumberFromFormat($format);

            if (empty($phoneNumber)) {
                continue;
            }

            $numberInfo = $this->getNumberInfoFromBase($phoneNumber, $numbersDataTableName, $needEncoding);

            $arNumbersWithInfo = array_merge($arNumbersWithInfo, $numberInfo);
        }

        return $arNumbersWithInfo;
    }

    /**
     * Получает информацию о номере из базы
     * @param int|string $phoneNumber
     * @return array
     * @throws \Radar\Exceptions\InvalidArgumentException Если в аргументе пусто
     */
    public function getNumberInfoFromBase($phoneNumber, $numbersDataTableName, $needEncoding = true)
    {
        if (empty($phoneNumber)) {
            throw new \Radar\Exceptions\InvalidArgumentException('Variable with phone number must not be empty !');
        }

        if (empty($numbersDataTableName)) {
            throw new \Radar\Exceptions\InvalidArgumentException('var with numbers data table name must not be empty !');
        }

        if (!is_string($numbersDataTableName)) {
            throw new \Radar\Exceptions\InvalidArgumentException('Var with numbers data table name must contain a string !');
        }

        $numbersTableName = $numbersDataTableName;

        $query = 'SELECT DISTINCT ContactId, Phone FROM calls WHERE ContactId = ' . $this->db->dbh->quote($phoneNumber);
        try {
            $res = $this->db->dbh->query($query);
        } catch (PDOException $e) {
            throw new \Radar\Exceptions\SelectDataException($e->getMessage(), $e->getTrace());
        }

        $result = $res->fetchAll();

        return $result;
    }

    /**
     * Изменяет кодировку во всех вложенных элементах массива 
     * @param array $dataToEncode Массив с данными для конвертации
     * @param mixed $encodeTo В какую кодировку конвертировать 
     * @param mixed $encodeFrom Из какой кодировки конвертировать 
     * @return array
     */
    public function encodeDataArrayAll($dataToEncode, $encodeTo = 'Windows-1251', $encodeFrom = 'UTF-8')
    {
        $encodedData = [];

        foreach ($dataToEncode as $rowIndex => $columns) {
            $encodedData[$rowIndex] = $this->encodeDataArrayRow($columns, $encodeTo, $encodeFrom);
        }

        return $encodedData;
    }

    /**
     * Изменяет кодировку в одном уровне массива
     * @param Массив с данными для конвертации $dataToEncode
     * @param mixed $encodeTo В какую кодировку конвертировать 
     * @param mixed $encodeFrom Из какой кодировки конвертировать 
     * @return array
     */
    public function encodeDataArrayRow($dataToEncode, $encodeTo = 'Windows-1251', $encodeFrom = 'UTF-8')
    {
        $encodedData = [];
        foreach ($dataToEncode as $fieldName => $data) {
            $encodedData[$fieldName] = mb_convert_encoding($data, $encodeTo, $encodeFrom);
        }

        return $encodedData;
    }

    /**
     * Получает номер телефона из строки или из массива
     * @param mixed $format
     * @return strin|int
     */
    private function getPhoneNumberFromFormat($format)
    {
        if (is_array($format)) {
            reset($format);
            return current($format);
        } elseif (is_string($format) || is_integer($format)) {
            return $format;
        }
    }

}
