<?php

namespace Radar\NumbersParser;

class SiteParser
{

    /**
     * Переменная с базовым url сайта
     * @var string 
     */
    private $baseSiteUrl;

    /**
     * Переменная с доплнительным url сайта
     * @var string 
     */
    private $additionalSiteUrl;

    /**
     * Расширение нужного файла
     * @var string 
     */
    private $neededFileExtension;

    /**
     * Ключевая фраза, если есть, будет учавствовать в regex'сe
     * @var string 
     */
    private $keyPhrase;

    /**
     * Исходная кодировка сайта
     * @var string 
     */
    private $sourceEncoding;

    /**
     * Путь к папке для загрузки файлов
     * @var string 
     */
    private $dowloadedFilesFolder;

    /**
     * 
     * @param string $baseSiteUrl
     * @param string $additionalSiteUrl
     * @param string $dowloadedFilesFolder
     * @param string $neededFileExtension
     * @param string $keyPhrase
     * @param string $sourceEncoding
     */
    public function __construct($baseSiteUrl, $additionalSiteUrl, $dowloadedFilesFolder, $neededFileExtension, $keyPhrase, $sourceEncoding = 'Windows-1251')
    {
        $this->baseSiteUrl = $baseSiteUrl;
        $this->additionalSiteUrl = $additionalSiteUrl;
        $this->siteUrl = $this->baseSiteUrl . $this->additionalSiteUrl;

        $this->dowloadedFilesFolder = $dowloadedFilesFolder;
        $this->neededFileExtension = $neededFileExtension;
        $this->keyPhrase = $keyPhrase;
        $this->sourceEncoding = $sourceEncoding;
    }

    /**
     * Возвращает контент страницы без разделения на строки
     * @return array
     */
    public function getSiteContent()
    {
        if (System::checkConsoleArg('-v')) {
            echo "Getting site content \n";
        }

        $siteContent = file_get_contents($this->siteUrl);

        if (System::checkConsoleArg('-v')) {
            echo "Done ! \n";
        }

        return $siteContent;
    }

    /**
     * Конвертирует кодировку сайта в UTF-8
     * @param string $siteContent
     * @return array
     */
    public function convertSiteEncoding($siteContent)
    {
        if (System::checkConsoleArg('-v')) {
            echo "Converting site content \n";
        }

        $convertedContent = \mb_convert_encoding($siteContent, 'UTF-8', $this->sourceEncoding);

        if (System::checkConsoleArg('-v')) {
            echo "Done !\n";
        }

        return $convertedContent;
    }

    /**
     * Получает дату последнего обновления списка номеров на сайте
     * @param string $convertedContent Конвертированный в юникод контент сайта
     * @return boolean|string
     */
    public function getNumbersDate($convertedContent)
    {
        $pattern = "([\d]{2}\.[\d]{2}\.[\d]{4})";

        if (!empty($this->keyPhrase)) {
            $pattern = "(?:$this->keyPhrase) " . $pattern;
        }

        $datesMtc = [];
        preg_match("#{$pattern}#", $convertedContent, $datesMtc);

        $matchedDate = $datesMtc[1];

        if (empty($matchedDate)) {
            die('Error! Cannot find numbers state date !');
            return false;
        }

        if (System::checkConsoleArg('-v')) {
            echo "Parsed numbers lists date is {$matchedDate}\n";
        }

        return $matchedDate;
    }

    /**
     * Возвращает список ссылок на файлы с списками номеров
     * @param string $unconvertedSiteContent Контент сайта в оригинальной кодировке
     * @return array
     */
    public function getNumbersFilesLinks($unconvertedSiteContent)
    {

// Попробуем распарсить DOM с помощью встроенной libxml-based библиотеки
        $dom = new \DOMDocument;
        $dom->loadHTML($unconvertedSiteContent);

        $possibleHrefs = [];
//Выберем все ссылки
        foreach ($dom->getElementsByTagName('a') as $link) {
            if ($link->hasAttribute('href')) {
                $href = $link->getAttribute('href');
//        Оставим только те, которые заканчиваются на нужное нам расширение файла
                if (strpos($href, $this->neededFileExtension, strlen($href) - strlen($this->neededFileExtension))) {
                    $possibleHrefs[] = $href;
                }
            }
        }

        if (empty($possibleHrefs)) {
            echo 'Cannot parse numbers files links !\n';
            return false;
        }

        return $possibleHrefs;
    }

    /**
     * Проверяет есть ли директория с новейшей датой, если нет - создает
     * @param string $newestNumbersDir Путь к директории с новейшей датой в названии
     * @return boolean
     */
    public function checkNumbersFilesDirs($newestNumbersDir)
    {
//  Если директория, переданная в аргументе существует - то у нас есть новейшая версия файлов с сайта
        if (is_dir($newestNumbersDir)) {
            if (System::checkConsoleArg('-v')) {
                echo "We have the latest version of numbers list files, using it !\n";
            }

            return false;
        } else {
            if (!mkdir($newestNumbersDir) && !is_dir($newestNumbersDir)) {
                die('Cannot create directory - ' . $newestNumbersDir);
            }
        }

        return true;
    }

    /**
     * Загружает файлы со списками номеров
     * @param array $numbersLinks Массив со ссылками
     * @param string $dirForDownloadedFiles Путь к директории для сохранения загруженных фалов
     * @return boolean
     */
    public function downloadNumbersListsFiles(array $numbersLinks, $dirForDownloadedFiles)
    {
        foreach ($numbersLinks as $href) {

            if (System::checkConsoleArg('-v')) {
                echo "Downloading {$href}\n";
            }

            $breakedHref = explode('/', $href);
            $fileName = '/' . $breakedHref[count($breakedHref) - 1];
            $downloadedFile = file_get_contents($this->baseSiteUrl . $href);
            $uploadPath = $dirForDownloadedFiles . $fileName;

            if (!file_put_contents($uploadPath, $downloadedFile)) {
                echo 'Cannot write or download file - ' . $uploadPath;
                return false;
            }

            if (System::checkConsoleArg('-v')) {
                echo "Successfully  saved in {$uploadPath}\n";
            }
        }

        return true;
    }

    /**
     *
     * @return type
     */
    public function getSiteContentByCURL()
    {
        $url = $this->siteUrl;
        $res = array();
        $options = array(
            CURLOPT_RETURNTRANSFER => true, // return web page
            CURLOPT_HEADER => false, // do not return headers
            CURLOPT_FOLLOWLOCATION => true, // follow redirects
            CURLOPT_USERAGENT => "spider", // who am i
            CURLOPT_AUTOREFERER => true, // set referer on redirect
            CURLOPT_CONNECTTIMEOUT => 120, // timeout on connect
            CURLOPT_TIMEOUT => 120, // timeout on response
            CURLOPT_MAXREDIRS => 10, // stop after 10 redirects
            CURLOPT_COOKIEJAR => './tmp/rcs_cookies.txt',
            CURLOPT_COOKIEFILE => './tmp/rcs_cookies.txt',
        );

        $ch = curl_init($url);
        curl_setopt_array($ch, $options);
        $content = curl_exec($ch);
        $err = curl_errno($ch);
        $errmsg = curl_error($ch);
        $header = curl_getinfo($ch);
        curl_close($ch);

        $res['content'] = $content;
        $res['url'] = $header['url'];
        return $res['content'];
    }

}
