<?php

namespace Radar\NumbersParser;

use PDO;

/**
 * Description of System
 *
 * @author Zelepukin
 */
class System
{

    /**
     * @var \Radar\DataBase\DataBase
     */
    private $db;

    /**
     * @var \Radar\NumbersParser\CsvParser 
     */
    private $csvParser;

    /**
     * Переменная с названием таблицы в базе для сохранения списков номеров
     * @type string
     */
    private $numbersDataTableName;

    /**
     * Перменная с названием таблицы с мета-данными в базе
     * @type string 
     */
    private $metadataTableName;

    /**
     * Массив с текущими аргументами командной строки
     * @type string
     */
    public static $consoleArgs = [];

    const CONSOLE_ARGUMENTS_LIST = [
        '-v' => 'verbose',
        '-s' => 'silent',
        '-fdu' => 'force numbers data update',
        '-ffu' => 'force numbers list files update'
    ];

    /**
     * 
     * @param \Radar\DataBase\DataBase $db
     * @param \Radar\NumbersParser\CsvParser $csvParser
     * @param string $numbersDataTableName
     * @param string $metadataTableName
     * @param string $consoleArguments
     */
    public function __construct(\Radar\DataBase\DataBase $db, CsvParser $csvParser, $numbersDataTableName, $metadataTableName, $consoleArguments = [])
    {
        $this->csvParser = $csvParser;
        $this->db = $db;
        $this->numbersDataTableName = $numbersDataTableName;
        $this->metadataTableName = $metadataTableName;
        self::$consoleArgs = $consoleArguments;
    }

    /**
     * Нужно ли обновлять списки номеров в базе ?
     * @param string $newestDirDate Название новейшей директории с файлами
     * @return boolean
     */
    public function isNumbersDataNeedUpdate($newestDirDate)
    {
//      Время последнего обновления берем из таблицы с мета-данными
        $lastUpdateDateUnix = strtotime($this->db->select(['last_update_date'], [$this->metadataTableName], false, false)['last_update_date']);
//      Ковертируем все это в юниксовое время и сравниваем 
        $newestDirDateUnix = strtotime($newestDirDate);

        if (($newestDirDateUnix > $lastUpdateDateUnix) || self::checkConsoleArg('-fdu')) {
            if (self::checkConsoleArg('-v') && !self::checkConsoleArg('-fdu')) {
                echo "Numbers data is deprecated, trying to update \n";
            } elseif (self::checkConsoleArg('-v') && !self::checkConsoleArg('-fdu')) {
                echo "Force update numbers data in base ! \n";
            }

            return true;
        }

        if (self::checkConsoleArg('-v')) {
            echo "Numbers data is up to date, skipping update \n";
        }

        return false;
    }

    /**
     * Переносит информацию по номерам из файлов в базу
     * @param array $filesList Массив с путями к файлам
     * @param boolean $truncateBefore Нужно ли очищать таблицу с данными до переноса
     * @return boolean
     */
    public function writeNumbersDataInBase(array $filesList, $truncateBefore = true)
    {
        if ($truncateBefore) {
            $this->truncateNumbersDataTable();
        }

        $sql = 'SHOW COLUMNS FROM ' . $this->numbersDataTableName;

        $columnRes = $this->db->dbh->query($sql);
        $columnNames = $columnRes->fetchAll(PDO::FETCH_COLUMN, 0);
// Ансетим id
        unset($columnNames[0]);

        $i = 0;
        $arPrepare = [];
		$columnNamesLength = count($columnNames);
        foreach ($filesList as $file) {
            $fileData = $this->csvParser->parseCsvFile($file);

            if (self::checkConsoleArg('-v')) {
                echo "Writing {$file} file data in base \n";
            }
	
            foreach ($fileData as $row => $columns) {
                $i++;
                if ($i > 1000) {
                    $this->db->insert($columnNames, $arPrepare, $this->numbersDataTableName);
                    $arPrepare = [];
                    $i = 0;
                }
                foreach ($columns as $columnIndex => $columnValue) {
					if($columnIndex > $columnNamesLength - 1){
						continue;
					}
					
                    $arPrepare[$row][$columnIndex] = $columnValue;
                }
            }
        }

        $this->updateLastUpdateDate();

        return true;
    }

    /**
     * Геттер для аргументов консоли
     * @return array
     */
    public function getConsoleArgs()
    {
        return $this->consoleArgs;
    }

    /**
     * Проверяет есть ли указанный аргумент в списке аргументов переданных из консоли
     * @param string $arg
     * @return boolean
     */
    public static function checkConsoleArg($arg)
    {
        if (in_array($arg, self::$consoleArgs)) {
            return true;
        }

        return false;
    }

    /**
     * Обновляет дату последнего обновления в таблице с мета-данными :)
     * @return boolean | int
     */
    private function updateLastUpdateDate()
    {
        return $this->db->update(['last_update_date' => date('Y-m-d')], [$this->metadataTableName]);
    }

    /**
     * Очищает таблицу с данными по номерам
     * @return boolean | int
     */
    private function truncateNumbersDataTable()
    {
        if (self::checkConsoleArg('-v')) {
            echo "Truncating numbers data table\n";
        }

        $sql = "TRUNCATE TABLE {$this->numbersDataTableName}";

        return $this->db->dbh->exec($sql);
    }

    /**
     * Сеттер для переменной с названием таблицы с информацией по номерам
     * @param string $numbersDataTableName Название таблицы
     * @throws \Radar\Exceptions\InvalidArgumentException Если в аргументе пусто
     */
    public function setNumbersDataTableName($numbersDataTableName)
    {
        if (empty($numbersDataTableName)) {
            throw new \Radar\Exceptions\InvalidArgumentException('Var with numbers data table name must not be empty !');
        }

        $this->numbersDataTableName = $numbersDataTableName;
    }

    /**
     * Геттер для переменной с названием таблицы с данными по номерам
     * @return string
     */
    public function getNumbersDataTableName()
    {
        return $this->numbersDataTableName;
    }

    /**
     * Удаляет из директории все файлы и папки
     * @param string $pathToDir
     * @return boolean
     */
    public function cleanDirectory($pathToDir)
    {
        if(!is_dir($pathToDir)){
            return false;
        }
        
        $files = glob($pathToDir.'*');
        foreach ($files as $file) {
            if (is_file($file)) {
                unlink($file);
            }
        }
        
        return true;
    }

}
